@@ -114,9 +114,9 @@ std::wstring FZ_PUBLIC_SYMBOL str_tolower_ascii(std::wstring_view const& s);
 std::string FZ_PUBLIC_SYMBOL str_toupper_ascii(std::string_view const& s);
 std::wstring FZ_PUBLIC_SYMBOL str_toupper_ascii(std::wstring_view const& s);
 
-/** \brief Comparator to be used for std::map for case-insentitive keys
+/** \brief Comparator to be used for std::map for case-insensitive keys
  *
- * Comparision is done locale-agnostic.
+ * Comparison is done locale-agnostic.
  * Useful for key-value pairs in protocols, e.g. HTTP headers.
  */
 struct FZ_PUBLIC_SYMBOL less_insensitive_ascii final
@@ -180,9 +180,12 @@ inline typename std::enable_if<std::is_arithmetic<std::decay_t<Arg>>::value, std
  *
  * \return the converted string on success. On failure an empty string is returned.
  */
-std::wstring FZ_PUBLIC_SYMBOL to_wstring_from_utf8(std::string const& in);
+std::wstring FZ_PUBLIC_SYMBOL to_wstring_from_utf8(std::string_view const& in);
 std::wstring FZ_PUBLIC_SYMBOL to_wstring_from_utf8(char const* s, size_t len);
 
+class buffer;
+std::wstring FZ_PUBLIC_SYMBOL to_wstring_from_utf8(fz::buffer const& in);
+
 /** \brief Converts from std::wstring into std::string in system encoding
  *
  * \return the converted string on success. On failure an empty string is returned.
@@ -283,12 +286,16 @@ template<> inline wchar_t const* choose_string(char const*, wchar_t const* w) {
 #endif
 
  /// Returns \c in with all occurrences of \c find in the input string replaced with \c replacement
-std::string FZ_PUBLIC_SYMBOL replaced_substrings(std::string const& in, std::string const& find, std::string const& replacement);
-std::wstring FZ_PUBLIC_SYMBOL replaced_substrings(std::wstring const& in, std::wstring const& find, std::wstring const& replacement);
+std::string FZ_PUBLIC_SYMBOL replaced_substrings(std::string_view const& in, std::string_view const& find, std::string_view const& replacement);
+std::wstring FZ_PUBLIC_SYMBOL replaced_substrings(std::wstring_view const& in, std::wstring_view const& find, std::wstring_view const& replacement);
+std::string FZ_PUBLIC_SYMBOL replaced_substrings(std::string_view const& in, char find, char replacement);
+std::wstring FZ_PUBLIC_SYMBOL replaced_substrings(std::wstring_view const& in, wchar_t find, wchar_t replacement);
 
 /// Modifies \c in, replacing all occurrences of \c find with \c replacement
-bool FZ_PUBLIC_SYMBOL replace_substrings(std::string& in, std::string const& find, std::string const& replacement);
-bool FZ_PUBLIC_SYMBOL replace_substrings(std::wstring& in, std::wstring const& find, std::wstring const& replacement);
+bool FZ_PUBLIC_SYMBOL replace_substrings(std::string& in, std::string_view const& find, std::string_view const& replacement);
+bool FZ_PUBLIC_SYMBOL replace_substrings(std::wstring& in, std::wstring_view const& find, std::wstring_view const& replacement);
+bool FZ_PUBLIC_SYMBOL replace_substrings(std::string& in, char find, char replacement);
+bool FZ_PUBLIC_SYMBOL replace_substrings(std::wstring& in, wchar_t find, wchar_t replacement);
 
 /**
  * \brief Tokenizes string.
@@ -520,6 +527,14 @@ bool ends_with(String const& s, String const& ending)
 	}
 }
 
+/**
+ * Normalizes various hyphens, dashes and minuses to just hyphen-minus.
+ *
+ * The narrow version assumes UTF-8 as encoding.
+ */
+std::string FZ_PUBLIC_SYMBOL normalize_hyphens(std::string_view const& in);
+std::wstring FZ_PUBLIC_SYMBOL normalize_hyphens(std::wstring_view const& in);
+
 }
 
 #endif
